--------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 1.0
--  \   \        Filename: $RCSfile: glb_dpm.vhd,v $           
--  /   /        Date Last Modified: $Date: 2008/07/02 10:57:32 $ 
-- /___/   /\    Date Created: April 2008 
-- \   \  /  \   
--  \___\/\___\
-- 
--
-------------------------------------------------------------------------------- 
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you 
-- a license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as is" solely for use in developing programs and 
-- solutions for Xilinx devices. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose. 
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications are
-- expressly prohibited.
-- 
--
-- Copyright (c) 2008 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
-- Description:
-- New Block Memory mini-core which instantiates the block memories for a
-- particular family.  This means we can avoid using inference code 
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

use work.global_types_pkg.all;

library XilinxCoreLib;
use XilinxCoreLib.blk_mem_gen_v2_7_comp.all;

entity glb_dpm is
  generic (
    width           : integer := 8;
    depth           : integer := 12;  
    delay           : integer := 3);  
  port (
    addra : in  std_logic_vector(depth-1 downto 0) := (others => '0');
    addrb : in  std_logic_vector(depth-1 downto 0) := (others => '0');
    clk   : in  std_logic                          := '1';
    wea   : in  std_logic                          := '1';
    ena   : in  std_logic                          := '1';
    enb   : in  std_logic                          := '1';
    dina  : in  std_logic_vector(width-1 downto 0) := (others => '0');
    doutb : out std_logic_vector(width-1 downto 0));

end entity glb_dpm;

architecture xilinx of glb_dpm is

signal wea_v : std_logic_vector(0 downto 0);

constant addr_depth : integer := 2**depth;
begin  -- architecture xilinx
     wea_v(0) <= wea;
     m1 : blk_mem_gen_v2_7
     generic map ( 
			            c_has_regceb 	         => 1,
			            c_has_regcea 	         => 1,
            			c_mem_type 		         => 2,
            			c_prim_type 	         => 1, 
            			c_disable_warn_bhv_coll=> 1, 
            			c_init_file_name 	     => "no_coe_file_loaded",
                  c_write_mode_a         => "READ_FIRST",
                  c_write_mode_b         => "READ_FIRST",
            			c_load_init_file 	     => 0,
                  c_write_depth_a        => addr_depth,
                  c_write_depth_b        => addr_depth,
			            c_read_depth_b 		     => addr_depth,
			            c_read_depth_a 		     => addr_depth,
			            c_write_width_b 	     => width,
			            c_write_width_a 	     => width,
			            c_read_width_b 	       => width,
			            c_read_width_a 	       => width,
			            c_algorithm 	         => 1, 
			            c_has_ssrb 		         => 0, 
			            c_has_ssra 		         => 0,
                  c_addra_width          => depth,
                  c_addrb_width          => depth,
            			c_disable_warn_bhv_range => 0, -- address out of range warnings
                  c_has_ena              => 1,
                  c_has_enb              => 1,
            			c_byte_size 		       => 9,
            			c_sim_collision_check  => "ALL",
            			c_common_clk 		       => 1,
            			c_wea_width 		       => 1,
            			c_web_width 		       => 1,
            			c_use_byte_web 		     => 0,
			            c_use_byte_wea 		     => 0,
            			c_use_default_data 	   => 0,
                  c_has_mem_output_regs_a => delay-1,
                  c_has_mem_output_regs_b => delay-1
                  )
     port map (dina   => dina,
               dinb   => (others => '0'),
               ena    => '1',
               enb    => enb,
               regcea => '1',
               regceb => enb,
               wea    => wea_v,
               web    => (others => '0'),
               clka   => clk,
               clkb   => clk,
               ssra   => '0',
               ssrb   => '0',
               addra  => addra,
               addrb  => addrb,
               douta  => open,
               doutb  => doutb);



end architecture xilinx;
