--------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 1.0
--  \   \        Filename: tb.vhd
--  /   /        Date Last Modified: 
-- /___/   /\    Date Created: August 2008 
-- \   \  /  \   
--  \___\/\___\
-- 
--
-------------------------------------------------------------------------------- 
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you 
-- a license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as is" solely for use in developing programs and 
-- solutions for Xilinx devices. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose. 
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications are
-- expressly prohibited.
-- 
--
-- Copyright (c) 2008 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE ieee.math_real.ALL;

--library UNISIM;
--use UNISIM.VCOMPONENTS.all;

LIBRARY work;
USE work.conversion_pkg.ALL;

entity awgn_gen is
  generic (
           output_rate     : INTEGER :=2;
           no_noise        : BOOLEAN := TRUE;
           widthd          : integer := 3;
           SNR             : real;
           Rate            : real;
           c_has_erased    : INTEGER := 0;
           c_soft_code     : INTEGER := 0 
           );
  port( ch_hard   : in  STD_LOGIC_VECTOR(output_rate-1 DOWNTO 0);
        erased    : in  STD_LOGIC_VECTOR(output_rate -1 DOWNTO 0);
        ce        : in  STD_LOGIC;
        reset     : in  STD_LOGIC;
        clk       : in  STD_LOGIC;
        ch_soft   : out viterbi_array
        );
end awgn_gen;

architecture behav of awgn_gen is
  TYPE  ch_r IS ARRAY (0 TO 7-1) OF REAL;
  TYPE  ch_i IS ARRAY (0 TO output_rate-1) OF INTEGER;

  signal EbNo     : real := 1.0;
  signal Ampl     : real := 1.0;
  signal qlevel   : real := 1.0;
  signal qlevelm1 : real := 1.0;
  signal qleveln  : real := 1.0;
  signal scal     : real := 1.0;
  
  signal ch_tx     : ch_r;
  signal ch_noise  : ch_r;
  signal ch_rcv    : ch_r;
  signal ch_scaled : ch_r;

  CONSTANT del : INTEGER := 3;  
  SIGNAL ch_hard_del : STD_LOGIC_VECTOR(output_rate -1 DOWNTO 0) := (OTHERS => '0');
  TYPE  ch_array IS ARRAY (0 TO output_rate-1) OF 
                    STD_LOGIC_VECTOR(widthd-1 DOWNTO 0); 
  TYPE  ch_del_array IS ARRAY (0 TO output_rate-1) OF 
                    STD_LOGIC_VECTOR(del-1 DOWNTO 0); 
 
  SIGNAL ch_soft_int : ch_array;
  
begin

  ampgen : process (EbNo, Ampl, qlevel)
  begin
    EbNo     <= (10.0**(SNR/10.0));
    Ampl     <= SQRT(2.0*EbNo*Rate);
    if(widthd > 1) then
       qlevel <= 2.0**(widthd - 1);  
    else
       qlevel <= 2.0;
    end if;   
    qlevelm1 <= qlevel - 1.0;
    qleveln  <= -1.0*qlevelm1;
    scal     <= qlevelm1/Ampl;
  end process ampgen;

  txgen : process (clk, reset)
  variable at_start : BOOLEAN := TRUE;
  begin
   if(at_start) then
      at_start := FALSE; 
      for i in 0 to output_rate -1 LOOP 
        ch_tx(i)   <= 1.0;
      end loop;   
   end if;
    if (clk = '1' and clk'event ) then
      if (reset = '1') then
        for i in 0 to output_rate -1 LOOP 
           ch_tx(i)   <= 0.0;
        end loop;   
      elsif (ce = '1') then
        for i in 0 to output_rate -1 LOOP 
           if (ch_hard(i) = '1') then
            ch_tx(i) <= -1.0*Ampl;
           else
            ch_tx(i) <= Ampl;
           end if;
        end loop;   
      end if;
    end if;
  end process txgen;

  noisgen : process (clk, reset)
    variable SEED1 : integer := 11;
    variable SEED2 : integer := 31;
    variable U1    : real;
    variable U2    : real;
    variable at_start : BOOLEAN := TRUE;
    variable cnt : integer := 0;
  begin
   if(at_start) then
      at_start := FALSE; 
      for i in 0 to output_rate -1 LOOP 
        ch_noise(i)   <= 1.0;
      end loop;   
   end if;
    if (clk = '1' and clk'event ) then
      if (reset = '1' ) then
        for i in 0 to output_rate -1 LOOP 
         ch_noise(i) <= 0.0;
        end loop; 
      elsif (ce = '1') then
--        cnt := cnt + 1;
--        if(cnt =2) then
--         cnt := 0;
--        end if; 
        for i in 0 to output_rate -1 LOOP 
           uniform(SEED1, SEED2, U1);
           uniform(SEED1, SEED2, U2);
           ch_noise(i) <= SQRT(-2.0*log(U1))*cos(2.0*MATH_PI*U2);
--           if(cnt = 0)then
--            ch_noise(i) <= 0.0;
--           end if;
        end loop; 
        uniform(SEED1, SEED2, U1); --!!!
        uniform(SEED1, SEED2, U2);
        ch_noise(0) <= SQRT(-2.0*log(U1))*cos(2.0*MATH_PI*U2);
--        if(cnt = 0)then
--          ch_noise(0) <= 0.0;
--        end if;
        IF (output_rate >1 ) THEN
           ch_noise(1) <= SQRT(-2.0*log(U1))*sin(2.0*MATH_PI*U2);
        END IF;
--        if(cnt = 0)then
--          ch_noise(1) <= 0.0;
--        end if;
      end if;
    end if;
  end process noisgen;

  rcvgen : process (clk, reset)
  variable at_start : BOOLEAN := TRUE;
  begin
   if(at_start) then
      at_start := FALSE; 
      for i in 0 to output_rate -1 LOOP 
        ch_rcv(i)   <= 1.0;
      end loop;   
   end if;
    if (clk = '1' and clk'event ) then
      if (reset = '1') then
        for i in 0 to output_rate -1 LOOP 
           ch_rcv(i) <= 0.0;
        end loop; 
      elsif (ce = '1') then
        for i in 0 to output_rate -1 LOOP 
          if(no_noise) then
            ch_rcv(i) <= ch_tx(i);
          else  
            ch_rcv(i) <= ch_tx(i) + ch_noise(i);
          end if;
        end loop; 
      end if;
    end if;
  end process rcvgen;

  scale : process (ch_rcv, scal)
  begin
    for i in 0 to output_rate -1 LOOP 
       ch_scaled(i) <= scal*ch_rcv(i);
    end loop;
  end process scale;

--  reg : process (clk, reset)
--  begin
--    if (clk = '1' and clk'event ) then
--      if (reset = '1') then
--        for i in 0 to output_rate -1 LOOP 
--           ch_real(i) <= 0.0;
--        end loop 
--      elsif (ce = '1') then
--        for i in 0 to output_rate -1 LOOP 
--           ch_real(i) <= ch_scaled(i);
--        end loop; 
--      end if;
--    end if;
--  end process reg;

  quant : process (clk, reset)
    variable ch_temp : ch_r;
    variable ch_del : ch_del_array;
    variable ch_conv : ch_i;
    constant convert : INTEGER := (2**(widthd-1)+1);
    constant convert1 : INTEGER := (2**(widthd-1)-1);
    variable at_start : BOOLEAN := TRUE;
  begin
   if(at_start) then
      at_start := FALSE; 
      for i in 0 to output_rate -1 LOOP 
        ch_del(i)   := (others => '0');
        ch_soft_int(i) <= (others => '0');
      end loop;   
   end if;
    if (clk = '1' and clk'event ) then
      if (reset = '1' ) then
        for i in 0 to output_rate -1 LOOP 
           ch_del(i) := (others => '0');
           ch_soft_int(i) <= (others => '0');
        end loop;
      elsif (ce = '1') then
        for i in 0 to output_rate -1 LOOP 
           ch_del(i)(del-1 DOWNTO 1) := ch_del(i)(del-2 DOWNTO 0); 
           ch_del(i)(0) :=  ch_hard(i);
        
           if (ch_scaled(i) > qlevelm1) then
             ch_temp(i) := qlevelm1;
           elsif (ch_scaled(i) <= qleveln) then
             ch_temp(i) := qleveln - 1.0;
           else
             ch_temp(i) := ch_scaled(i);
           end if;
           ch_hard_del(i) <= ch_del(i)(del-1);
           -- convert integers to correct format
           ch_conv(i) := integer(floor(ch_temp(i)));
           if(widthd = 1) then
            -- hard coding
              if(ch_conv(i) < 0) then
                  ch_conv(i) := 1;
              else
                  ch_conv(i) := 0;
              end if; 
           else
              if( c_soft_code = soft_signed_magnitude ) then
                 if(ch_conv(i) < 0) then
                    ch_conv(i) := -ch_conv(i) - convert;
                 end if;       
              else
                 if(ch_conv(i) < 0) then
                    ch_conv(i) := -ch_conv(i) - convert;
                 end if;       
                 if(ch_conv(i) >= 0) then
                    ch_conv(i) := -ch_conv(i) + convert1;
                 end if;       
              end if;
           end if; 
           IF(c_has_erased /= 0) THEN 
              IF(erased(i) = '1') THEN
               ch_conv(i) := 0;
              END IF;
           END IF;
           ch_soft_int(i) <= to_vector(widthd, ch_conv(i));
        end loop;   
      end if;
    end if;
  end process quant;

  gen_out:
  for i in 0 to output_rate - 1 generate
     ch_soft(i)(widthd-1 DOWNTO 0) <= ch_soft_int(i)(widthd-1 DOWNTO 0);
 end generate;   
end behav;
