--------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 1.0
--  \   \        Filename: tb.vhd
--  /   /        Date Last Modified: 
-- /___/   /\    Date Created: August 2008 
-- \   \  /  \   
--  \___\/\___\
-- 
--
-------------------------------------------------------------------------------- 
--Legal Disclaimer
--
--(c) Copyright 2002 - 2009 Xilinx, Inc. All rights reserved. 
--<Enter range of dates starting with first year core released>
--
--This file contains confidential and proprietary information
--of Xilinx, Inc. and is protected under U.S. and
--international copyright and other intellectual property
--1aws.
----
--DISCLAIMER
--This disclaimer is not a license and does not grant any
--rights to the materials distributed herewith. Except as
--otherwise provided in a valid license issued to you by
--Xilinx, and to the maximum extent permitted by applicable
--law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
--WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
--AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
--BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
--INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
--(2) Xilinx shall not be liable (whether in contract or tort,
--including negligence, or under any other theory of
--liability) for any loss or damage of any kind or nature
--related to, arising under or in connection with these
--materials, including for any direct, or any indirect,
--special, incidental, or consequential loss or damage
--(including loss of data, profits, goodwill, or any type of
--loss or damage suffered as a result of any action brought
--by a third party) even if such damage or loss was
--reasonably foreseeable or Xilinx had been advised of the
--possibility of the same.
----
--CRITICAL APPLICATIONS
--Xilinx products are not designed or intended to be fail-
--safe, or for use in any application requiring fail-safe
--performance, such as life-support or safety devices or
--systems, Class III medical devices, nuclear facilities,
--applications related to the deployment of airbags, or any
--other applications that could lead to death, personal
--injury, or severe property or environmental damage
--individually and collectively, "Critical
--Applications"). Customer assumes the sole risk and
--liability of any use of Xilinx products in Critical
--Applications, subject only to applicable laws and
--regulations governing limitations on product liability.
----
--THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
--PART OF THIS FILE AT ALL TIMES.
--------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

package conversion_pkg is

  type viterbi_array      is array (0 to 2) of std_logic_vector(4 downto 0); 
  constant soft_signed_magnitude   : integer := 0;
  constant soft_offset_binary      : integer := 1;
  constant soft_signed             : integer := 2;

  
  function to_uint (a : std_logic_vector) return integer;

  function to_vector (size : integer; val : integer) return std_logic_vector;

end conversion_pkg;


package body conversion_pkg is


-------------------------------------------------------------------------------
-- Convert a std_logic_vector to an unsigned integer
-------------------------------------------------------------------------------
  function to_uint (a : std_logic_vector) return integer is

    alias av     : std_logic_vector (1 to a'length) is a;
    variable val : integer := 0;
    variable b   : integer := 1;

  begin

    for i in a'length downto 1 loop

      if (av(i) = '1') then             -- if LSB is '1',
        val := val + b;                 -- add value for current bit position
      end if;

      b := b * 2;                       -- Shift left 1 bit

    end loop;
    return val;

  end to_uint;

-------------------------------------------------------------------------------
-- Convert an integer to a std_logic_vector
-------------------------------------------------------------------------------
  function to_vector (size : integer; val : integer) return std_logic_vector is

    variable vec      : std_logic_vector (1 to size);
    variable a        : integer;
    variable negative : boolean := false;

  begin

    negative := (val < 0);
    a        := val;

    for i in size downto 1 loop
      if ((a mod 2) = 1) then
        vec(i) := '1';
      else
        vec(i) := '0';
      end if;

      a := a / 2;

    end loop;

    if negative then
      vec := not(vec) + '1';
    end if;

    return vec;

  end to_vector;

end conversion_pkg;
